use super::{get_player_item_or_fail, user_error, UResult, UserVerb, UserVerbRef, VerbContext};
use crate::{
    models::item::LocationActionType,
    regular_tasks::queued_command::{
        queue_command_and_save, QueueCommand, QueueCommandHandler, QueuedCommandContext,
    },
    services::comms::broadcast_to_room,
};
use async_trait::async_trait;
use std::time;

pub struct QueueHandler;
#[async_trait]
impl QueueCommandHandler for QueueHandler {
    async fn start_command(&self, ctx: &mut QueuedCommandContext<'_>) -> UResult<time::Duration> {
        if ctx.item.death_data.is_some() {
            user_error(
                "You try to stand, but it turns out the dead can't even do that!".to_owned(),
            )?;
        }
        match ctx.command {
            QueueCommand::Stand => {}
            _ => user_error("Unexpected command".to_owned())?,
        };
        match ctx.item.action_type {
            LocationActionType::Sitting { .. } | LocationActionType::Reclining { .. } => {}
            _ => user_error("You're already standing.".to_owned())?,
        }
        Ok(time::Duration::from_secs(1))
    }

    #[allow(unreachable_patterns)]
    async fn finish_command(&self, ctx: &mut QueuedCommandContext<'_>) -> UResult<()> {
        if ctx.item.death_data.is_some() {
            user_error(
                "You try to stand, but it turns out the dead can't even do that!".to_owned(),
            )?;
        }
        match ctx.command {
            QueueCommand::Stand => {}
            _ => user_error("Unexpected command".to_owned())?,
        };
        match ctx.item.action_type {
            LocationActionType::Sitting { .. } | LocationActionType::Reclining { .. } => {}
            _ => user_error("You're already standing.".to_owned())?,
        }
        let msg_exp = format!(
            "{} stands up.\n",
            &ctx.item.display_for_sentence(true, 1, true),
        );
        let msg_nonexp = format!(
            "{} stands up.\n",
            &ctx.item.display_for_sentence(false, 1, true),
        );
        broadcast_to_room(
            ctx.trans,
            &ctx.item.location,
            None,
            &msg_exp,
            Some(&msg_nonexp),
        )
        .await?;

        ctx.item.action_type = LocationActionType::Normal;
        Ok(())
    }
}

pub struct Verb;
#[async_trait]
impl UserVerb for Verb {
    async fn handle(
        self: &Self,
        ctx: &mut VerbContext,
        _verb: &str,
        _remaining: &str,
    ) -> UResult<()> {
        let player_item = get_player_item_or_fail(ctx).await?;

        if player_item.death_data.is_some() {
            user_error(
                "You try to stand, but it turns out the dead can't even do that!".to_owned(),
            )?;
        }

        queue_command_and_save(ctx, &player_item, &QueueCommand::Stand).await?;
        Ok(())
    }
}
static VERB_INT: Verb = Verb;
pub static VERB: UserVerbRef = &VERB_INT as UserVerbRef;
